

#include <stdio.h>
#include "avr_uart_i.h"

#if SIMAVR
static int uart_putchar(char c, FILE *stream)
{
    static char last_c;
    if (c == '\n' && last_c != '\r') {
        uart_putchar('\r', stream);
    }
    last_c = c;
    loop_until_bit_is_set(UCSR0A, UDRE0);
    UDR0 = c;
    return 0;
}

static FILE uart_stream =
    FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

void avr_setup(void)
{
    stdout = &uart_stream;
    stderr = &uart_stream;
}

#else

static int _put(char c, FILE *x)
{
    static uint8_t r = 0;
    (void)x;
    r = (c == '\r');
    if (c == '\n' && !r) {
        uart0_putc('\r');
    }
    uart0_putc(c);
    return 0;
}

static int _get(FILE *x)
{
    (void)x;
    return (int)uart0_getc();
}

static FILE uart_stream = FDEV_SETUP_STREAM(_put, _get, _FDEV_SETUP_RW);

void avr_setup(void)
{
    uart0_init();
    stdin = &uart_stream;
    stdout = &uart_stream;
    stderr = &uart_stream;
}

#endif
