/* gage_std.c */
/*
    This file is part of lwc-gage-avr.
    Copyright (C) 2019 bg nerilex (bg@nerilex.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "gage_std.h"

void Initialize(struct state *s) {
    int i;

    /* We initialize the state with zeros */
    /* should be with memset() for slightly better performance */
    for (i = 0; i < INTERNAL_STATE_SIZE_BYTES; ++i) {
        s->a[i] = 0;
    }
}

void Padd(unsigned char *mpad, const unsigned char *in, unsigned long long inlen) {
    unsigned long long i, lastbytes;
    int j;

    /* determine how many bytes from the original message go in the mpad block */
    lastbytes = inlen % RATE_BYTES;
    /* and put them in mpad */
    j = 0;
    for (i = inlen - lastbytes; i < inlen; ++i) {
        mpad[j++] = in[i];
    }

    /* append the byte 0x80 */
    mpad[j++] = 0x80;

    /* if there are still some remaining bytes in mpad, set them to 0 */
    while (j<RATE_BYTES) mpad[j++] = 0;
}

void Absorb(struct state *s, const unsigned char *in, unsigned long long inlen, unsigned char *mpad) {
    unsigned long long i, lastblockbyteindex;
    int j;

    lastblockbyteindex = inlen - (inlen % RATE_BYTES);
    i = 0;
    while (i < lastblockbyteindex) {
        for (j = 0; j < RATE_BYTES; ++j)
            s->a[j] ^= in[i++];
        GAGE_CORE(s);
    }
    /* absorb the final mpad part */
    for (j = 0; j < RATE_BYTES; ++j)
        s->a[j] ^= mpad[j];
    GAGE_CORE(s);
}

void Squeeze(struct state *s, unsigned char *out) {
    int i, j;

    i = 0;
    while (i < CRYPTO_BYTES) {
        for (j = 0; j < RATE_BYTES; ++j)
            out[i++] = s->a[j];
        GAGE_CORE(s);
    }
}
