/* performance_test.h */
/*
    Copyright (C) 2006-2015 bg nerilex (bg@nerilex.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PERFORMANCE_TEST_H_
#define PERFORMANCE_TEST_H_

#include <stdint.h>
#include <avr/pgmspace.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#define START_TIMER (TCCR1B = 1)
#define STOP_TIMER (TCCR1B = 0)

#if SIMAVR

#ifdef ATMEGA644
#define SIMAVR_COM_R GPIOR0
#endif

#ifdef ATMEGA128
#define SIMAVR_COM_R EEARL
#endif

void simavr_set_label_P(PGM_P s);

#define calibrateTimer(a)
#define startTimer(a)  /* do {     GPIOR0 = 0; } while(0) */
#define stopTimer(a) 0 /* (GPIOR0 = 1, 0) */

#define getOverhead(a, b)

#define print_time_P(a, b) ((void)t) /* avoid possible warning of unused variable */

#define print_overhead(a)

#if 0
#define MEASURE_b(x)                                                           \
    do {                                                                       \
        static const char label[] PROGMEM = #x;                                \
        simavr_set_label_P(label);                                             \
        x;                                                                     \
    } while (0)
#else
#define MEASURE_b(x)                                                             \
    do {                                                                       \
        static const char label[] PROGMEM = #x;                                \
        simavr_set_label_P(label);                                             \
        SIMAVR_COM_R = 0;                                                            \
        x;                                                                     \
        SIMAVR_COM_R = 1;                                                            \
    } while (0)

#endif

#define MEASURE(x)                                                             \
    do {                                                                       \
        static const char label[] PROGMEM = #x;                                \
        simavr_set_label_P(label);                                             \
        SIMAVR_COM_R = 0;                                                            \
        x;                                                                     \
        SIMAVR_COM_R = 1;                                                            \
    } while (0)

#define MEASURE_x(x)                                                           \
    do {                                                                       \
        char *label = #x;                                                      \
        while (*label) {                                                       \
            SIMAVR_COM_R = *label++;                                                 \
        }                                                                      \
        SIMAVR_COM_R = 0;                                                            \
        x;                                                                     \
        SIMAVR_COM_R = 1;                                                            \
    } while (0)

#else

#define simavr_set_label_P(a)
#define MEASURE(x) x
#define MEASURE_b(x) MEASURE(x)
void calibrateTimer(void);
void startTimer(uint8_t granularity);
uint32_t stopTimer(void);
void getOverhead(uint16_t *constoh, uint16_t *intoh);

void print_time_P(PGM_P s, uint32_t t);
void print_overhead(void);

#endif

#endif /*PERFORMANCE_TEST_H_*/
