/* gage_core_232.S */
/*
    This file is part of lwc-gage-avr.
    Copyright (C) 2019 bg nerilex (bg@nerilex.org)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

.nolist
#include <avr/io.h>
#include "parameters.h"
#include "avr-asm-macros.S"
.list

/*
 * this version is much like asm1, but instead of calling a function,
 * the functions are 'inlined' manually.
 */

STATE_B = INTERNAL_STATE_SIZE_BYTES


/*
 * aligns to a 16-byte boundary,
 * which allows adding an index into the table using OR
 */
.align 4
quas:
/* Quasigroups Nr. 173 (in lexicigraphical order of all 576 4x4 quasigroups) */
	.byte 1, 0, 3, 2
	.byte 0, 2, 1, 3
	.byte 2, 3, 0, 1
	.byte 3, 1, 2, 0

.align 4
masks:
	.byte 0x10
	.byte 0x08
	.byte 0x04
	.byte 0x20
	.byte 0x01
	.byte 0x40
	.byte 0x02
	.byte 0x80

/******************************************************************************/

tmp_a = 0
backup_y = 18
tmp_m = 20
tmp_b = 21
ctr = 22
buf_low_limit = 23
buf_idx = 24
buf = 28

tmp_ax = 25
tmp_bx = 24
tmp_cx = 0
tmp_dx = 23
ctr = 22
ctr2 = 21

s_backup = 14
s_backup_lo = 14
s_backup_hi = 15
round = 16
lfsr = 17

.global qpermutation
.global gage_core_232

gage_core_232:
	ldi r22, ROUNDS
; next comes the start of qpermutation, so we can skip the call (or jump)

; state pointer is passed in r24:r25
; rounds is passed in r22

; NOTE: LFSR computation works only for an EVEN NUMBER of ROUNDS!
qpermutation:
; we need to backup some registers
	push_ lfsr, round
	movw backup_y, Y
	stack_alloc STATE_B, r28, r29
	adiw Y, 1

	mov round, r22
	movw X, r24
	ldi lfsr, 0xAA

30:
	clr r22
	ror lfsr
	sbc r22, r22 ; this should not modify the carry
	rol r24
	lsl r24
	bst round, 0
	bld r24, 0
	andi r22, 0xB8
	eor lfsr, r22

dtransform:
	ldi r30, lo8(quas)
	ldi r31, hi8(quas)
	ldi ctr, STATE_B

1:
	clr tmp_dx
	ld tmp_ax, X
	ldi ctr2, 4
2:
	andi tmp_bx, 0x03 ; leaders are in b
	rol tmp_ax        ; new byte from state is in a
	rol tmp_bx
	rol tmp_ax
	rol tmp_bx
	or r30, tmp_bx
	lpm tmp_cx, Z
	andi r30, 0xF0
	lsl tmp_dx
	lsl tmp_dx
	or tmp_dx, tmp_cx
	dec ctr2
	brne 2b

	st X+, tmp_dx
	dec ctr
	brne 1b
	sbiw X, STATE_B

; MAIN LOOP CONTROL
	dec round
	breq 40f ; exit function

10:
shuffle_state_bits:
	mov buf_low_limit, r28
	movw buf_idx, Y

; zero temporary buffer
	ldi ctr, STATE_B
1:
	st Y+, r1
	dec ctr
	brne 1b

; BEGIN main loop
	ldi r30, lo8(masks)
	ldi r31, hi8(masks)
	ldi ctr, STATE_B
1:
	ld tmp_a, X+
	adiw buf_idx, 1
	movw Y, buf_idx

2:
	ld tmp_b, -Y
	lpm tmp_m, Z+
	lsr tmp_a
	sbc r1, r1
	and r1, tmp_m
	or tmp_b, r1
	clr r1
	st Y, tmp_b

; BEGIN handle wrap around in the state
	cpse r28, buf_low_limit
	rjmp 3f
	adiw Y, STATE_B
3:
; END handle wrap around in the state

	cpi tmp_m, 0x80
	brne 2b
	andi r30, 0xF0
;	sbiw Z, 8 ; if not aligned

	dec ctr
	brne 1b
	adiw Y, 8
; END main loop

9:
; copy data from temporary buffer into state
	ldi ctr, STATE_B
1:
	ld r0, -Y
	st -X, r0
	dec ctr
	brne 1b

; epilogue
	rjmp 30b

40:

	movw Y, backup_y
	stack_free STATE_B

	pop_ round, lfsr
	ret
